/*
  KISC-32 Compiler | main.cpp
  Copyright (C) 2024  Kaelan Carlos

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.

  Contact email: kelna@kelna.dev
*/

#include <iostream>
#include <filesystem>

#include "tokens/tokeniser.hpp"
#include "messages.hpp"

int main(int argc, char *argv[]) {
  // for string comparisons >_<
  using namespace std::literals;

  // note to self: the program itself is an arg, change at your own peril
  if (argc < 2) {
    std::cout << msgs::welcome << std::endl;
    return EXIT_SUCCESS;
  }

  int flag_count = 0;
  bool summary = false;
  bool verbose = false;
  std::string in_file_name = "";
  std::string out_file_name = "a.bisc";

  // parse cli args
  for (int i = 1; i < argc; i++) {
    // FLAGS
    if (argv[i][0] == '-') {
      std::string flag(argv[i]); 
      flag_count++;
      
      if (flag == "-h"sv || flag == "--help"sv) {
        std::cout << msgs::help << std::endl;
        return EXIT_SUCCESS;
      }
      
      else if (flag == "-c"sv || flag == "--copyright"sv) {
        std::cout << msgs::copyright << std::endl;
        return EXIT_SUCCESS;
      }
      
      else if (flag == "-v"sv || flag == "--verbose"sv) {
        verbose = true;
        summary = true;
      }
      
      else if (flag == "-s"sv || flag == "--summary"sv) {
        summary = true;
      }
      
      else if (flag == "-o"sv || flag == "--output") {
        i++;
        if (i == argc) {
          msgs::error("missing filename after '\033[1m-o\033[0m'");
        }
        out_file_name = argv[i];
      }
      
      else {
        msgs::warn("unknown option '\033[1m" + flag + "\033[0m' has been ignored");
      }
    }

    // FILENAME
    else {
      std::string arg(argv[i]);

      if (in_file_name != ""sv)
        msgs::warn("extraneous argument '\033[1m" + arg + "\033[0m' has been ignored");

      in_file_name = arg;
    }
  }

  // Sanitise and validate file name
  if (in_file_name == ""sv) {
    msgs::fatal_error("no input files");
  } else if (!std::filesystem::exists(in_file_name)) {
    msgs::fatal_error(in_file_name + ": No such file or directory");
  }
  
  if (summary) {
    std::cout << in_file_name << " found, starting compilation." << std::endl;
    std::cout << out_file_name << " will be the final binary." << std::endl;
  }

  return EXIT_SUCCESS;
}