/*
  KISC-32 Compiler | messages.hpp
  Copyright (C) 2024  Kaelan Carlos

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.

  Contact email: kelna@kelna.dev
*/

#include <iostream>


namespace msgs
{
// MESSAGES
std::string copyright = R"(KISC-32 Compiler | messages.hpp
Copyright (C) 2024  Kaelan Carlos

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

Contact email: kelna@kelna.dev)";

std::string welcome = "Welcome to the KISC-32 Compiler. To get started, add --help or -h.";

std::string help = R"(Usage: k32c [options] file...

Options:
  --copyright, or -c    Display this program's copyright information.
  --help, or -h         Display this information.
  --summary, or -s      Display summarised output during compilation.
  --verbose, or -v      Display more thorough output during compilation.)";

std::string warning_prefix = "\033[1mk32c: \033[35mwarning: \033[0m";
std::string error_prefix = "\033[1mk32c: \033[31merror: \033[0m";
std::string fatal_error_prefix = "\033[1mk32c: \033[31mfatal error: \033[0m";


// HELPER FUNCTIONS
void warn(std::string err_msg) {
  std::cout << warning_prefix;
  std::cout << err_msg;
  std::cout << std::endl;
}

void error(std::string err_msg) {
  std::cout << error_prefix;
  std::cout << err_msg;
  std::cout << std::endl;
  std::cout << "Compilation terminated." << std::endl;
  exit(EXIT_FAILURE);
}

void fatal_error(std::string err_msg) {
  std::cout << fatal_error_prefix;
  std::cout << err_msg;
  std::cout << std::endl;
  std::cout << "Compilation terminated." << std::endl;
  exit(EXIT_FAILURE);
}
}